# Generated by Django 4.2.5 on 2023-09-23 12:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("users", "0003_alter_customuser_managers"),
    ]

    operations = [
        migrations.AlterField(
            model_name="customuser",
            name="date_of_birth",
            field=models.DateField(blank=True, null=True, verbose_name="Date of Birth"),
        ),
    ]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import Column, Field, Layout, Row, Submit
from django import forms
from django.contrib.auth import authenticate, login
from django.contrib.auth.forms import AuthenticationForm, UserChangeForm, UserCreationForm
from django.urls import reverse

from .models import CustomUser

GENDER_CHOICES = (
    ("", "Choose..."),
    ("M", "Male"),
    ("F", "Female"),
    ("O", "Other"),
)
CONTACT_CHOICES = (
    ("", "Choose..."),
    ("P", "Phone"),
    ("E", "Email"),
    ("N", "None"),
)


class CustomUserCreationForm(UserCreationForm):
    first_name = forms.CharField(
        label="First Name",
        widget=forms.TextInput(attrs={"class": "form-control", "placeholder": "Enter First Name"}),
        required=True,
    )
    last_name = forms.CharField(
        label="Last Name",
        widget=forms.TextInput(attrs={"class": "form-control", "placeholder": "Enter Last Name"}),
        required=True,
    )
    email = forms.CharField(
        widget=forms.EmailInput(attrs={"class": "form-control", "placeholder": "eg. example@example.com"}),
        required=True,
    )
    phone = forms.CharField(
        widget=forms.NumberInput(attrs={"class": "form-control", "placeholder": "eg. +977-98XXXXXXXX"}),
        required=True,
    )
    gender = forms.ChoiceField(
        choices=GENDER_CHOICES,
        required=True,
    )
    address = forms.CharField(
        widget=forms.TextInput(attrs={"class": "form-control", "placeholder": "eg. Street 1, City"}),
        required=True,
    )
    nationality = forms.CharField(
        widget=forms.TextInput(attrs={"class": "form-control", "placeholder": "eg. Nepali"}),
        required=True,
    )
    date_of_birth = forms.DateField(
        label="Date of Birth",
        widget=forms.TextInput(attrs={"type": "date"}),
        required=True,
    )
    education = forms.CharField(
        widget=forms.TextInput(attrs={"class": "form-control", "placeholder": "eg. Bachelors"}),
        required=True,
    )
    contact_mode = forms.ChoiceField(
        label="Contact Mode Preference",
        choices=CONTACT_CHOICES,
        required=True,
    )
    # password = forms.CharField(widget=forms.PasswordInput(attrs={"class": "form-control"}))

    class Meta:
        model = CustomUser

        fields = (
            "first_name",
            "last_name",
            "email",
            "phone",
            "gender",
            "address",
            "nationality",
            "date_of_birth",
            "education",
            "contact_mode",
            "password1",
            "password2",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        # self.helper.form_tag = False
        # self.helper.form_method = "POST"
        # self.helper.form_action = 'signup'
        # self.helper.form_action = reverse("home")
        self.helper.form_show_errors = True
        self.helper.layout = Layout(
            Row(
                Column("first_name", prepend_text="@", css_class="form-group col-md-6 mb-0"),
                Column("last_name", css_class="form-group col-md-6 mb-0"),
                css_class="form-row",
            ),
            Row(
                Column("date_of_birth", css_class="form-group col-md-6 mb-0"),
                Column(Field("gender", css_class="form-control col-md-6 mb-0")),
                css_class="form-row",
            ),
            Row(
                Column("email", css_class="form-group col-md-6 mb-0"),
                Column("phone", css_class="form-group col-md-6 mb-0"),
                css_class="form-row",
            ),
            Row(
                Column("nationality", css_class="form-group col-md-6 mb-0"),
                Column("address", css_class="form-group col-md-6 mb-0"),
                css_class="form-row",
            ),
            "education",
            Row(
                Column(Field("contact_mode", css_class="form-control mb-0 col-md-6")),
            ),
            Row(
                Column(Field("password1", css_class="form-control mb-0 col-md-6")),
                Column(Field("password2", css_class="form-control mb-0 col-md-6")),
            ),
            Submit("submit", "Sign Up"),
        )

    def clean_email(self):
        email = self.cleaned_data.get("email")
        print(email)

        if CustomUser.objects.filter(email=email).exists():
            raise forms.ValidationError("This email is already in use.")
        return email

    def save(self, commit=True):
        user = super().save(commit=False)
        user.username = self.cleaned_data["email"]
        if commit:
            user.save()
        return user


class CustomUserChangeForm(UserChangeForm):
    class Meta:
        model = CustomUser

        fields = (
            "first_name",
            "last_name",
            "email",
            "phone",
            "gender",
            "address",
            "nationality",
            "date_of_birth",
            "education",
            "contact_mode",
            "password",
        )


class CustomAuthenticationForm(AuthenticationForm):
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop("request", None)
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.form_show_errors = True
        self.helper.layout = Layout(
            "username",
            "password",
            Submit("submit", "login"),
        )

    username = forms.EmailField(label="Username/Email", widget=forms.TextInput(attrs={"class": "form-control"}))
    password = forms.CharField(widget=forms.PasswordInput(attrs={"class": "form-control"}))

    class Meta:
        fields = (
            "username",
            "password",
        )

    def clean(self):
        cleaned_data = super().clean()
        print(cleaned_data)
        email = cleaned_data.get("username")
        password = cleaned_data.get("password")

        if email and password:
            # Authenticate using the email (username) and password
            self.user_cache = authenticate(username=email, password=password)
            if self.user_cache is None or not self.user_cache.is_active:
                raise forms.ValidationError("Invalid email or password")
        return self.cleaned_data



# Generated by Django 4.2.5 on 2023-09-24 11:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("users", "0004_alter_customuser_date_of_birth"),
    ]

    operations = [
        migrations.AddField(
            model_name="customuser",
            name="archive",
            field=models.BooleanField(default=False, verbose_name="Archive"),
        ),
    ]



from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.views import LoginView
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django.urls import reverse, reverse_lazy
from django.views.decorators.cache import cache_control
from django.views.generic import TemplateView, View
from django.views.generic.edit import CreateView

from .forms import CustomAuthenticationForm, CustomUserCreationForm
from .models import CustomUser


class MyLoginView(LoginView):
    form_class = CustomAuthenticationForm
    template_name = "users/login.html"
    # success_url = reverse_lazy("home")

    def form_valid(self, form):
        user = form.user_cache
        print(user)

        if user and user.is_active:
            print("Authenticated user:", user)
            print("Redirecting to 'home'")
            login(self.request, user)
            return redirect("users:home")
        else:
            print("User authentication failed or user is inactive")
            return self.form_invalid(form)

    def get_success_url(self):
        return reverse("users:home")  # Ensure 'home' is the correct URL name

    def form_invalid(self, form):
        print("Form is invalid")  # Check if this is called for invalid forms
        print(form.user_cache)
        return super().form_invalid(form)


class SignUp(CreateView):
    model = CustomUser
    form_class = CustomUserCreationForm
    template_name = "users/create.html"
    success_url = reverse_lazy("users:user_login")

    def form_valid(self, form):
        form.instance.username = form.cleaned_data["email"]
        return super().form_valid(form)


def dashboard(request):
    return render(request, "home.html", {"data": "some text based data"})


@cache_control(no_cache=True, must_revalidate=True, no_store=True)
def user_logout(request):
    logout(request)
    return redirect("users:user_login")



# Generated by Django 4.2.5 on 2023-09-23 12:25

from django.db import migrations
import users.managers


class Migration(migrations.Migration):
    dependencies = [
        ("users", "0002_alter_customuser_email"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="customuser",
            managers=[
                ("objects", users.managers.CustomUserManager()),
            ],
        ),
    ]



from django.contrib.auth.models import AbstractUser, PermissionsMixin
from django.db import models

from .managers import CustomUserManager

# Backward compatible with django 3.x
try:
    from django.utils.translation import ugettext_lazy as _
except ImportError:
    from django.utils.translation import gettext_lazy as _


class CustomUser(AbstractUser):
    GENDER_CHOICES = (
        ("M", "Male"),
        ("F", "Female"),
        ("O", "Other"),
    )
    CONTACT_CHOICES = (
        ("P", "Phone"),
        ("E", "Email"),
        ("N", "None"),
    )

    first_name = models.CharField(_("First Name"), max_length=50)
    last_name = models.CharField(_("Last Name"), max_length=50)
    gender = models.CharField(_("Gender"), max_length=50, choices=GENDER_CHOICES)
    phone = models.CharField(_("Phone"), max_length=50)
    email = models.EmailField(_("Email Address"), unique=True, max_length=254)
    address = models.CharField(_("Address"), max_length=254)
    nationality = models.CharField(_("Nationality"), max_length=50)
    date_of_birth = models.DateField(_("Date of Birth"), auto_now=False, auto_now_add=False, blank=True, null=True)
    education = models.CharField(_("Education Background"), max_length=50)
    contact_mode = models.CharField(_("Prefered Contact Mode"), max_length=50, choices=CONTACT_CHOICES)
    is_admin = models.BooleanField(_("Admin"), default=False)
    archive = models.BooleanField(_("Archive"), default=False)

    objects = CustomUserManager()

    USERNAME_FIELD = "email"

    REQUIRED_FIELDS = []

    def __str__(self):
        return f"{self.first_name} {self.last_name} || {self.email}"



from rest_framework import serializers

from .models import CustomUser


class CustomUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = CustomUser
        fields = "__all__"



# Generated by Django 4.2.5 on 2023-09-23 12:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("users", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="customuser",
            name="email",
            field=models.EmailField(
                max_length=254, unique=True, verbose_name="Email Address"
            ),
        ),
    ]



from django.urls import path

from .views import MyLoginView, SignUp, dashboard, user_logout

app_name = "users"

urlpatterns = [
    path("register/", SignUp.as_view(), name="user_register"),
    path("login/", MyLoginView.as_view(), name="user_login"),
    path("", dashboard, name="home"),
    path("logout", user_logout, name="user_logout"),
]



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin

from .forms import CustomUserChangeForm, CustomUserCreationForm
from .models import CustomUser


class CustomUserAdmin(UserAdmin):
    add_form = CustomUserCreationForm
    form = CustomUserChangeForm

    model = CustomUser
    list_display = [
        "email",
        "first_name",
        "last_name",
        "phone",
        "is_active",
        "is_admin",
    ]
    list_filter = (
        "is_active",
        "is_admin",
    )
    ordering = ("-date_joinded",)

    fieldsets = (
        (
            "Basic Information",
            {
                "fields": (
                    "first_name",
                    "last_name",
                    "email",
                    "phone",
                    "address",
                    "nationality",
                    "date_of_birth",
                    "education",
                    "contact_mode",
                )
            },
        ),
        (
            "Permissions",
            {
                "fields": (
                    "is_staff",
                    "is_admin",
                    "is_active",
                )
            },
        ),
    )
    add_fieldsets = (
        (
            "Basic Information",
            {
                "classes": ("wide",),
                "fields": (
                    "first_name",
                    "last_name",
                    "email",
                    "phone",
                    "address",
                    "nationality",
                    "date_of_birth",
                    "education",
                    "contact_mode",
                    "password1",
                    "password2",
                ),
            },
        ),
    )
    search_fields = (
        "email",
        "first_name",
        "last_name",
    )
    ordering = ("email",)


admin.site.register(CustomUser, CustomUserAdmin)
